# -*- coding: UTF-8 -*-
###########################################################################
#
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# libsecure.py
#
# classes utilitaires pour lancement des services en https
#
###########################################################################

import os
from OpenSSL import SSL
from tempfile import mktemp
from creole.creolecat import instance
from creole.parsedico import parse_dico
from creole.config import cert_file, key_file

def gen_certif(certfile=None):
    """génère un certificat pour le server https
    """
    if certfile is None:
        certfile = cert_file
        keyfile = key_file
        requete_dir = os.path.dirname(certfile)
    else:
        requete_dir = os.path.dirname(certfile)
        basefile = os.path.splitext(os.path.basename(certfile))[0]
        keyfile = "%s/%s.key" % (requete_dir, basefile)
    if os.path.isfile(certfile):
        # certif déjà présent
        return True
    if not os.path.isdir(requete_dir):
        os.makedirs(requete_dir)

    conf_template = '/usr/share/creole/certif-eole.conf'
    conf_file = '/etc/sysconfig/eole/certif-eole.conf'
    if not os.path.isfile(conf_template):
        print "Fichier template de la configuration du certificat non trouvé :\n\n\t%s\n" % conf_template
        return False
    # instanciation du template
    instance(conf_template, None, conf_file, '/etc/eole/config.eol')
    if not os.path.isfile(conf_file):
        print 'Erreur de génération du fichier de configuration ssl'
        return False
    tmp_keyfile = mktemp()

    # génération certificat x509
    ssl_cmd = "openssl req -x509 -new -newkey rsa:1024 -days 1825 -config %s -keyout %s -out %s >/dev/null" % (conf_file, tmp_keyfile, certfile)
    res = os.system(ssl_cmd)
    if res != 0:
        print '\n! erreur lors de la génération du certificat ssl dans %s !\n' % certfile
        return False

    # génération de la clé publique
    key_cmd = "openssl rsa -in %s -passin pass:secret -out %s >/dev/null" % (tmp_keyfile, keyfile)
    res = os.system(key_cmd)
    if res != 0:
        print('\n! erreur lors de la génération de la clé ssl dans %s !\n' % keyfile)
    os.unlink(tmp_keyfile)
    # finalisation du fichier certificat
    dico = parse_dico()
    cert_data = open(certfile).read()
    key_data = open(keyfile).read()
    try:
        f_cert = open(certfile,'w')
        f_cert.write("SubjectAltName=%s.%s\n%s\n%s" % (dico['nom_machine'], dico['nom_domaine_local'],cert_data.rstrip(),key_data.rstrip()))
        f_cert.close()
    except:
        print "\n! Erreur lors de l'écriture du fichier %s" % certfile

    return True
